import json

from xml.dom.minidom import parseString

from django.core.files.base import ContentFile


def bake(imageFile, assertion_string):
    svg_doc = parseString(imageFile.read())
    imageFile.close()

    assertion_node = svg_doc.createElement('openbadges:assertion')
    assertion_node = _populate_assertion_node(assertion_node, assertion_string)

    svg_body = svg_doc.getElementsByTagName('svg')[0]
    svg_body.setAttribute('xmlns:openbadges', "http://openbadges.org")
    svg_body.insertBefore(assertion_node, svg_body.firstChild)

    new_file = ContentFile(svg_doc.toxml('utf-8'))
    new_file.close()
    return new_file


def _populate_assertion_node(assertion_node, assertion_string):
    try:
        assertion = json.loads(assertion_string)
    except ValueError:
        assertion = None

    if assertion:
        verify_url = assertion.get('verify', {}).get('url')
        if verify_url:
            assertion_node.setAttribute('verify', verify_url)
        else:
            # TODO: Support 0.5 badges
            pass
        character_data = svg_doc.createCDATASection(assertion_string)
        assertion_node.appendChild(character_data)

    else:
        assertion_node.setAttribute('verify', assertion_string)

    return assertion_node


def unbake(imageFile):
    svg_doc = parseString(imageFile.read())

    assertion_node = svg_doc.getElementsByTagName("openbadges:assertion")[0]
    character_data = None
    for node in assertion_node.childNodes:
        if node.nodeType == node.CDATA_SECTION_NODE:
            character_data = node.nodeValue
    url = assertion_node.attributes['verify'].nodeValue.encode('utf-8')
    return character_data or url



import re
from xml.dom.minidom import parseString

import png_bakery


def check_image_type(imageFile):
    if imageFile.read(8) == '\x89PNG\r\n\x1a\n':
        return 'PNG'
    imageFile.seek(0)
    # TODO: Use xml library to more accurately detect SVG documents
    if re.search('<svg', imageFile.read(256)):
        return 'SVG'

def unbake(imageFile):
    """
    Return the openbadges content contained in a baked image.
    """
    image_type = check_image_type(imageFile)
    imageFile.seek(0)
    if image_type == 'PNG':
        return png_bakery.unbake(imageFile)
    elif image_type == 'SVG':
        return svg_bakery.unbake(imageFile)

def bake(imageFile, assertion_json_string):
    """
    Embeds a serialized representation of a badge instance in an image file.
    """
    image_type = check_image_type(imageFile)
    imageFile.seek(0)
    if image_type == 'PNG':
        return png_bakery.bake(imageFile, assertion_json_string)
    elif image_type == 'SVG':
        return svg_bakery.bake(imageFile, assertion_json_string)



import hashlib
import itertools
import os.path
import png
import re

from django.core.files.base import ContentFile


def unbake(imageFile):
    """
    Return the openbadges content contained in a baked PNG file.
    If this doesn't work, return None.

    If there is both an iTXt and tEXt chunk with keyword openbadges,
    the iTXt chunk content will be returned.
    """

    reader = png.Reader(file=imageFile)
    for chunktype, content in reader.chunks():
        if chunktype == 'iTXt' and content.startswith('openbadges\x00'):
            return re.sub('openbadges[\x00]+', '', content)
        elif chunktype == 'tEXt' and content.startswith('openbadges\x00'):
            return content.split('\x00')[1]


def bake(imageFile, assertion_json_string):
    """
    Embeds a serialized representation of a badge instance in a PNG image file.
    """
    reader = png.Reader(file=imageFile)

    output_filename = '%s.png' % hashlib.md5(str(assertion_json_string)).hexdigest()

    newfile = ContentFile("", name=output_filename)
    newfile.open()
    chunkheader = 'openbadges\x00\x00\x00\x00\x00'
    badge_chunk = ('iTXt', bytes(chunkheader + assertion_json_string))
    png.write_chunks(newfile, baked_chunks(reader.chunks(), badge_chunk))

    newfile.close()
    return newfile


def baked_chunks(original_chunks, badge_chunk):
    """
    Returns an iterable of chunks that places the Open Badges baked chunk
    and filters out any previous Open Badges chunk that may have existed.
    """
    def is_not_previous_assertion(chunk):
        if chunk[1].startswith('openbadges\x00'):
            return False
        return True

    first_slice = itertools.islice(original_chunks, 1)
    last_slice = itertools.ifilter(
        is_not_previous_assertion,
        itertools.islice(original_chunks, 1, None)
    )

    return itertools.chain(first_slice, [badge_chunk], last_slice)
